#include <avr/io.h>
#include <util/delay.h>
#include "twi_master.h"

#define PORT_CTRL PORTC
#define DDR_CTRL DDRC

#define BIT_DATA  PC0
#define BIT_LATCH PC1
#define BIT_CLK   PC2


#define PULSE_CLK() {_delay_us(10); PORT_CTRL |= (1<<BIT_CLK); _delay_us(10); PORT_CTRL &= ~(1<<BIT_CLK); }
#define PULSE_LATCH() {_delay_us(10); PORT_CTRL |= (1<<BIT_LATCH); _delay_us(10); PORT_CTRL &= ~(1<<BIT_LATCH); }

#define SET_DATA() {PORT_CTRL |= (1<<BIT_DATA);}
#define CLEAR_DATA() {PORT_CTRL &= ~(1<<BIT_DATA);}

#define WAIT_TICK() {while((TIFR & (1<<OCF2)) == 0);TIFR |=(1<<OCF2);}

void init(){
	DDR_CTRL |= (1<<BIT_DATA) | (1<<BIT_LATCH) | (1<<BIT_CLK);

	TCCR2 = (1<<WGM21) | 0b111;//CTC mode, clk/1024
	OCR2 = 140;

	DDRB &= ~(1<<DDB3);
	PORTB|= (1<<PB3);
	DDRB &= ~(1<<DDB4);
        PORTB |= (1<<PB4);
	DDRD &= ~(1<<DDD0);
        PORTD |= (1<<PD0);
	DDRD &= ~(1<<DDD1);
        PORTD |= (1<<PD1);
}

uint8_t tr_table[10] = {8,6,4,2,0,1,3,5,7,9};

void put_digit(uint8_t digit){

	uint8_t tr = tr_table[digit];
	uint8_t x;
	for(x=0;x<10;x++){
		if(x==tr){
			SET_DATA();
		}else{
			CLEAR_DATA();
		}
		PULSE_CLK();
	}

}

void put_digits(uint8_t * digits){
	uint8_t x;
	
	for(x=0;x<6;x++){
		put_digit(digits[x]);
	}
	if (digits[6]==0) {
		for (x=0;x<4;x++) {
			CLEAR_DATA();
			PULSE_CLK();
		}
	}
	else {
                for (x=0;x<4;x++) {
                        SET_DATA();
                        PULSE_CLK();
                }
	}
	PULSE_LATCH();
}


void ds1307_write(uint8_t addr, uint8_t data){
	TWIM_Start(0xd0, 0);
	TWIM_Write(addr);
	TWIM_Write(data);
	TWIM_Stop();
}

uint8_t ds1307_read(uint8_t addr){
	uint8_t data;
	
	//set addr
	TWIM_Start(0xd0, 0);
	TWIM_Write(addr);
	TWIM_Stop();
	
	//read data addr points to
	TWIM_Start(0xd0, 1);
	data = TWIM_ReadNack();
	TWIM_Stop();
	
	return data;

}

void ds1307_init(){
	TWIM_Init();
	_delay_ms(1);
	ds1307_write(0,0x00);
	ds1307_write(1,0x37);
	ds1307_write(2,0x04);
}

void set_time(){
	uint8_t x,min,h;
	h = ds1307_read(2);
	min = ds1307_read(1);
	x=0;
	while(1){
		if ( ! (PINB & (1 << PB3)) ) {
			// tmp = ds1307_read(2);
			if (h >= 0x23) {
				h=0x00;
			}
			else {
                        	h++;
			}
                        // ds1307_write(2,tmp);
                        _delay_ms(100);
			x=0;
                }
		if ( ! (PIND & (1 << PD1)) ) {
			if (h <= 0x00 ) {
				h = 0x23;
			}
			else {
                        	h--;
			}
                        _delay_ms(100);
			x=0;
		}
                if ( ! (PINB & (1 << PB4)) ) {
			if (min >= 0x59) {
                                min=0x00;
                        }
                        else {
                                min++;
                        }
                        _delay_ms(100);
			x=0;
                }
                if ( ! (PIND & ( 1 << PD0 )) ) {
			if (min <= 0x00 ) {
                                min = 0x59;
                        }
                        else {
                                min--;
                        }
                        _delay_ms(100);
                        x=0;
                }
		_delay_ms(100);
		x++;
		if (x>=63) {
			ds1307_write(1,min);
			ds1307_write(2,h);
			break; 
		}
		
	}
}
void set_alarm(){
	while(1){
		break;
	}
}

int main(){
	uint8_t x;
	x=0;
	uint8_t digits[7];
	digits[6] = 0;
	init();
	ds1307_init();
	while(1) {
		uint8_t tmp;
		if ( ! (PINB & (1 << PB3)) ) {
			set_time();
		}
		if ( ! (PIND & (1 << PD1)) ) {
			set_alarm();
                }
		tmp = ds1307_read(0);
		digits[5] = tmp & 0x0f;
		digits[4] = tmp >> 4;
		tmp = ds1307_read(1);
		digits[3] = tmp & 0x0f;
		digits[2] = tmp >> 4;
		tmp = ds1307_read(2);
		digits[1] = tmp & 0x0f;
		digits[0] = tmp >> 4;
		x++;
		if (x >= 63) {
			if (digits[6]==0) {
				digits[6] = 1;
			} else {
				digits[6] = 0;
			}
			x=0;
		}
		put_digits(digits);
		WAIT_TICK();
	}
}
